<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    cloudsme-prototype
 * @subpackage cloudsme-prototype/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    cloudsme-prototype
 * @subpackage cloudsme-prototype/includes
 * @author     Your Name <email@example.com>
 */
class Cloudsme_Prototype {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Cloudsme_Prototype_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {

        $this->plugin_name = 'cloudsme-prototype';
        $this->version = '1.0.0';

        $this->load_dependencies();
        $this->set_locale();
        if (is_admin() != false) {
            $this->define_admin_hooks();
        } else {
            $this->define_public_hooks();
        }
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Cloudsme_Prototype_Loader. Orchestrates the hooks of the plugin.
     * - Cloudsme_Prototype_i18n. Defines internationalization functionality.
     * - Cloudsme_Prototype_Admin. Defines all hooks for the admin area.
     * - Cloudsme_Prototype_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cloudsme-prototype-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cloudsme-prototype-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-cloudsme-prototype-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-cloudsme-prototype-public.php';

        /**
         * Iinclude authorization widget
         */
        require plugin_dir_path(dirname(__FILE__)) . 'includes/class-cloudsme-prototype-widget.php';


        $this->loader = new Cloudsme_Prototype_Loader();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Plugin_Name_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Cloudsme_Prototype_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Cloudsme_Prototype_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

        // register widget
        add_action('widgets_init', create_function('', 'return register_widget("cloudsme_widget");'));

        //add session
        add_action('init', array($this, 'my_session_start'), 1);
        add_action('wp_logout', array($this, 'my_session_end'));

        //ajax
        add_action('wp_enqueue_scripts', array($this, 'process_ajax_post_script'));
        add_action('wp_ajax_nopriv_process_ajax_post', array($this, 'process_ajax_post_func'));
        add_action('wp_enqueue_scripts', array($this, 'process_ajax_upload_script'));
        add_action('wp_ajax_nopriv_process_ajax_upload', array($this, 'process_ajax_upload_func'));
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Cloudsme_Prototype_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
        $this->loader->add_action('admin_menu', $plugin_admin, 'create_admin_menu_page');

        //test
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'settings_selection_script');
        $this->loader->add_action('wp_ajax_settings_selection', $plugin_admin, 'settings_selection_func');

        // register widget
        add_action('widgets_init', create_function('', 'return register_widget("cloudsme_widget");'));

        //ajax
        add_action('wp_enqueue_scripts', array($this, 'process_ajax_post_script'));
        add_action('wp_ajax_process_ajax_post', array($this, 'process_ajax_post_func'));
        add_action('wp_enqueue_scripts', array($this, 'process_ajax_upload_script'));
        add_action('wp_ajax_process_ajax_upload', array($this, 'process_ajax_upload_func'));

        add_action('wp_enqueue_scripts', array($this, 'process_ajax_post_script'));
        add_action('wp_ajax_nopriv_process_ajax_post', array($this, 'process_ajax_post_func'));
        add_action('wp_enqueue_scripts', array($this, 'process_ajax_upload_script'));
        add_action('wp_ajax_nopriv_process_ajax_upload', array($this, 'process_ajax_upload_func'));

        //session
        add_action('init', array($this, 'my_session_start'), 1);
        add_action('wp_logout', array($this, 'my_session_end'));
    }

    // ajax-related
    public function process_ajax_post_script() {
        wp_enqueue_script('process_ajax_post', plugin_dir_url(__FILE__) . '../admin/js/process_ajax_post.js', array('jquery'));
        wp_localize_script('process_ajax_post', 'PT_Ajax', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'nextNonce' => wp_create_nonce('myajax-next-nonce')
                )
        );
    }

    public function process_ajax_upload_script() {
        wp_enqueue_script('process_ajax_upload', plugin_dir_url(__FILE__) . '../admin/js/process_ajax_file_upload.js', array('jquery'));
        wp_localize_script('process_ajax_upload', 'CSME_upload', array(
            'ajax_url' => admin_url('admin-ajax.php')
                )
        );
    }

    public function process_ajax_upload_func() {
        $response_data = array();
        $response_data['launched'] = 'false';

        $createJobResult = $this->createJob();
        if ($createJobResult['create_res'] == 'true') {
            $uploadFileResult = $this->uploadFile();

            if ($uploadFileResult['add_file_res'] == 'true') {
                $startJobResult = $this->startJob();

                if ($startJobResult['submit_res'] == 'true') {
                    $response_data['launched'] = 'true';
                } else {
                    // failed to start the job
                    $response_data['startJobResult'] = $startJobResult['Curlgetinfo'];
                }
            } else {
                // file upload failed
                $response_data['uploadFileResult'] = $uploadFileResult['Curlgetinfo'];
            }
        } else {
            // job creation failed
            $response_data['createJobResult'] = $createJobResult['Curlgetinfo'];
        }

        //echo $response_data;

        $response = json_encode($response_data);
        header("Content-Type: application/json");
        echo $response;
        exit;
    }

    private function uploadFile() {

        $ft_name = $_FILES["file"]["name"];
        $ft_type = $_FILES["file"]["type"];
        $ft_path = $_FILES["file"]["tmp_name"];
        $ft_error = $_FILES["file"]["error"];
        $ft_size = $_FILES["file"]["size"];

        $response_data = array();

        $job_id = $_SESSION['job_id_s'];
        $data_type = (string) get_option('o_input_type');
        ($_POST['archive'] == 'true') ? $is_archive = '1' : $is_archive = '0';
        $data_type_id = $this->get_data_type_id($data_type);

        // initialise the curl request
        $url = get_option('o_platform_url') . "/data_files";
        $request = curl_init($url);

        curl_setopt($request, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
        curl_setopt($request, CURLOPT_HTTPHEADER, array("Content-Type:multipart/form-data"));
        curl_setopt($request, CURLOPT_POST, true);
        curl_setopt(
                $request, CURLOPT_POSTFIELDS, array(
            'data' => $this->initCURLfile($ft_path, $ft_type, $ft_name),
            "archive" => $is_archive,
            "data_type_id" => $data_type_id,
            "job_id" => $job_id
        ));

        curl_setopt($request, CURLOPT_RETURNTRANSFER, true);

        curl_exec($request);

        if (curl_errno($request) == 0) {
            $response_data['add_file_res'] = true;
        } else {
            $response_data['add_file_res'] = false;
            $response_data['Curlgetinfo'] = curl_getinfo($request);
        }

        curl_close($request);

        $response = json_encode($response_data);
        header("Content-Type: application/json");
        //echo $response;
        return $response_data;
        //exit();
    }

    private function process_ajax_upload_func_old() {

        $ft_name = $_FILES["file"]["name"];
        $ft_type = $_FILES["file"]["type"];
        $ft_path = $_FILES["file"]["tmp_name"];
        $ft_error = $_FILES["file"]["error"];
        $ft_size = $_FILES["file"]["size"];

        $response_data = array();

        $job_id = $_SESSION['job_id_s'];
        $data_type = (string) get_option('o_input_type');
        ($_POST['archive'] == true) ? $is_archive = "true" : $is_archive = "false";
        $data_type_id = $this->get_data_type_id($data_type);

        // initialise the curl request
        $url = get_option('o_platform_url') . "/data_files.xml";
        $request = curl_init($url);

        curl_setopt($request, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
        curl_setopt($request, CURLOPT_HTTPHEADER, array("Content-Type:multipart/form-data"));
        curl_setopt($request, CURLOPT_POST, true);
        curl_setopt(
                $request, CURLOPT_POSTFIELDS, array(
            'data' => $this->initCURLfile($ft_path, $ft_type, $ft_name),
            "archive" => $is_archive,
            "data_type_id" => $data_type_id,
            "job_id" => $job_id
        ));

        curl_setopt($request, CURLOPT_RETURNTRANSFER, true);

        curl_exec($request);

        if (curl_errno($request) == 0) {
            $response_data['add_file_res'] = true;
        } else {
            $response_data['add_file_res'] = false;
            $response_data['Curlgetinfo'] = curl_getinfo($request);
        }

        curl_close($request);

        $response = json_encode($response_data);
        header("Content-Type: application/json");
        echo $response;
        exit();
    }

    private function initCURLfile($filename, $contentType, $postName) {
        // PHP 5.5
        if (function_exists('curl_file_create')) {
            return curl_file_create($filename, $contentType, $postName);
        }
        // Use the old style if using an older version of PHP
        $value = "@{$filename};filename=" . $postName;
        if ($contentType) {
            $value .= ';type=' . $contentType;
        }

        return $value;
    }

    public function process_ajax_post_func() {
        $nonce = $_POST['nextNonce'];
        if (!wp_verify_nonce($nonce, 'myajax-next-nonce')) {
            die('Nonce verification failed');
        }

        $command = $_POST['command'];
        $response_data = array();

        if ($command) {
            switch ($command) {

                case 'login':
                    $response_data['login_res'] = $this->login($_POST['username'], $_POST['password']);
                    if ($response_data['login_res']) {
                        list ($u_name, $trash) = explode('@', $_SESSION['username_s'], 2);

                        $software = $this->getSoftwareName();
                        $_SESSION['software_val'] = $software;
                        $_SESSION['name_val'] = $u_name;


                        $response_data['name'] = $u_name;
                        $response_data['soft_name'] = $software;

                        $_SESSION['csme_step_num'] = "2";
                        //setcookie("csme_step_num", $_SESSION['csme_step_num'], null, COOKIEPATH, COOKIE_DOMAIN);
                    } else {
                        $response_data['name'] = "guest";
                    }

                    break;
                case 'logout':
                    if ($_REQUEST['founds_status'] == 'true') {
                        $cancel_job = $this->cancel_job();
                    }
                    $response_data['logout_res'] = $this->logout_actions();


                    break;
                case 'restore_state':
                    $response_data = $this->restoreState();

                    break;
                case 'get_files':

                    $response_data = $this->getDataFilesList();
                    $_SESSION['csme_step_num'] = "4"; //if success

                    break;
                case 'refresh_status':
                    $response_data['job_status'] = $this->refresh_status();
                    $response_data['job_name'] = $_SESSION['job_name_s'];

                    break;
                case 'next_job':
                    $response_data['run_next'] = $this->run_next_preparing();

                    break;
                case 'fdw_triggered':
                    $response_data = $this->get_link_to_file($_POST['fdw_id']);

                    break;
                case 'cancel_button_cb':
                    $response_data = $this->cancel_job();

                    break;
            }
        } else {
            $response_data = 'unknown command';
        }

        $response = json_encode($response_data);
        header("Content-Type: application/json");
        echo $response;
        exit;
    }

    private function restoreState() {
        $values = Array();

        //After successful step number of next one stores in session.
        //Each step requires specific options to be restored.

        $state = $_SESSION["csme_step_num"];
        switch ($state) {
            case "1":
                $values["csme_step_num"] = "1";
                //$values["csme_sid"] = $_SESSION['csme_sid'];

                break;

            case "2":
                $values["csme_step_num"] = "2";
                $values['csme_uname'] = $_SESSION['name_val'];
                $values['csme_sname'] = $_SESSION['software_val'];
                //$values["csme_sid"] = $_SESSION['csme_sid'];

                break;

            case "3":
                $values["csme_step_num"] = "3";
                //$values["csme_sid"] = $_SESSION['csme_sid'];
                break;

            case "4":
                $values["csme_step_num"] = "4";
                //$values["csme_sid"] = $_SESSION['csme_sid'];
                break;

            default:
                $values["csme_step_num"] = "ERROR " . $state;
            //$values["csme_sid"] = $_SESSION['csme_sid'];
        }
        return $values;
    }

    private function logout_actions() {
        //session_destroy();

        if (isset($_SESSION['username_s']))
            unset($_SESSION['username_s']);
        if (isset($_SESSION['password_s']))
            unset($_SESSION['password_s']);
        if (isset($_SESSION['jobs_array_s']))
            unset($_SESSION['jobs_array_s']);
        if (isset($_SESSION['data_file_array_s']))
            unset($_SESSION['data_file_array_s']);
        if (isset($_SESSION['job_id_s']))
            unset($_SESSION['job_id_s']);
        if (isset($_SESSION['job_name_s']))
            unset($_SESSION['job_name_s']);
        if (isset($_SESSION['uid_s']))
            unset($_SESSION['uid_s']);
        //state related
        if (isset($_SESSION['csme_step_num']))
            unset($_SESSION['csme_step_num']);
        if (isset($_SESSION['name_val']))
            unset($_SESSION['name_val']);
        if (isset($_SESSION['software_val']))
            unset($_SESSION['software_val']);
        if (isset($_SESSION['csme_sid']))
            unset($_SESSION['csme_sid']);

        $_SESSION['csme_step_num'] = "1";

        return true;
    }

    private function run_next_preparing() {

        if (isset($_SESSION['jobs_array_s']))
            unset($_SESSION['jobs_array_s']);
        if (isset($_SESSION['data_file_array_s']))
            unset($_SESSION['data_file_array_s']);
        if (isset($_SESSION['job_id_s']))
            unset($_SESSION['job_id_s']);
        if (isset($_SESSION['job_name_s']))
            unset($_SESSION['job_name_s']);

        $_SESSION['csme_step_num'] = "2";

        return true;
    }

    private function refresh_status() {
        if (empty($_SESSION['job_id_s'])) {
            return "job not exist";
        } else {
            $url = get_option('o_platform_url') . "/jobs.xml?job[user_filter]=" . $_SESSION['uid_s'];

            $status = "udefined";
            $process = curl_init($url);

            curl_setopt($process, CURLOPT_HEADER, 1);
            curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
            curl_setopt($process, CURLOPT_TIMEOUT, 30);
            curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

            $return = curl_exec($process);

            $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
            $header = substr($return, 0, $header_size);
            $body = substr($return, $header_size);

            curl_close($process);

            $xml = simplexml_load_string($body);

            foreach ($xml->job as $job) {
                if ($job->id == $_SESSION['job_id_s']) {

                    $status = (string) $job->status;
                }
            }
            if ($status == 'submitted') {
                $url = get_option('o_platform_url') . "/jobs/" . $_SESSION['job_id_s'] . "/activity_logs.xml";


                $process = curl_init($url);


                curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
                curl_setopt($process, CURLOPT_TIMEOUT, 30);
                curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

                $return = curl_exec($process);


                curl_close($process);

                $xml = simplexml_load_string($return);
                $xml = (array) $xml;
                foreach ($xml['activity-log'] as $value) {
                    $value = (array) $value;
                    if ($value['activity-code'] == "174") {

                        return 'Not enough funds';
                    }
                }
            }
            return $status;
        }
    }

    private function getDataFilesList() {
        $url = get_option('o_platform_url') . "/jobs/" . $_SESSION['job_id_s'] . "/data_files.xml";
        $dfArray = array();

        $process = curl_init($url);
        curl_setopt($process, CURLOPT_HEADER, 1);
        curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
        $return = curl_exec($process);
        $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
        $body = substr($return, $header_size);
        curl_close($process);

        $xml = simplexml_load_string($body);

        $number = 0;
        foreach ($xml->{'data-file'} as $data_file) {
            $number++;
            $idattr = (string) $data_file->id;
            $df_key = "data_file" . $number;
            $df_value = (string) $data_file->{'data-file-name'} . "," . $idattr;
            $dfArray[$df_key] = $df_value;
        }

        $_SESSION['csme_step_num'] = "4";

        return $dfArray;
    }

    private function getSoftwareName() {
        $soft_name = "undefined";
        $soft_id = get_option('o_software');

        $url = get_option('o_platform_url') . "/softwares.xml";
        $process = curl_init($url);

        curl_setopt($process, CURLOPT_HEADER, 1);
        curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

        $return = curl_exec($process);
        $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
        $header = substr($return, 0, $header_size);
        $body = substr($return, $header_size);
        curl_close($process);

        $xml = simplexml_load_string($body);

        foreach ($xml->{'software'} as $software) {
            if ($software->{'id'} == $soft_id) {
                $soft_name = (string) ($software->name);
            }
        }

        return $soft_name;
    }

    private function get_link_to_file($data_file_id) {

        if (!empty($data_file_id)) {
            $url = get_option('o_platform_url') . "/data_files/" . (string) $data_file_id . "/download.xml";
            $body = Array();

            $process = curl_init($url);

            curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
            curl_setopt($process, CURLOPT_CUSTOMREQUEST, "PUT");
            curl_setopt($process, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($process, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
            curl_setopt($process, CURLOPT_POSTFIELDS, http_build_query($body));
            curl_setopt($process, CURLOPT_VERBOSE, 1);

            $response_data = curl_exec($process);

            /*        if(curl_errno($process)){
              $response_data['Curl_error'] = curl_error($process);
              }
              $response_data['Curl_get_info'] = curl_getinfo($process); */

            curl_close($process);

            //$link_to_file = $response_data;
            return $response_data;
        } else {
            return "empty_arg_error";
        }
    }

    private function get_data_type_id($data_type_name) {
        if (!empty($data_type_name)) {
            $url = get_option('o_platform_url') . "/data_types.xml";
            $process = curl_init($url);

            curl_setopt($process, CURLOPT_HEADER, 1);
            curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
            curl_setopt($process, CURLOPT_TIMEOUT, 30);
            curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

            $return = curl_exec($process);
            $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
            $header = substr($return, 0, $header_size);
            $body = substr($return, $header_size);
            curl_close($process);

            $xml = simplexml_load_string($body);

            $data_type_id = "";

            foreach ($xml->{'data-type'} as $data_type) {
                if (!empty($data_type->{'name'}) && ((string) $data_type->{'name'} == $data_type_name )) {
                    $data_type_id = (string) ($data_type->id);
                }
            }

            return $data_type_id;
        } else {
            return "empty_arg_error";
        }
    }

    private function startJob() {

        $job_id = $_SESSION['job_id_s'];
        $url = get_option('o_platform_url') . "/jobs/" . $job_id . "/submit.xml";
        $response_data = Array();
        $body = Array();

        $request = curl_init($url);

        curl_setopt($request, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
        curl_setopt($request, CURLOPT_CUSTOMREQUEST, "PUT");
        curl_setopt($request, CURLOPT_TIMEOUT, 30);
        curl_setopt($request, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($request, CURLOPT_POSTFIELDS, http_build_query($body));

        $return = curl_exec($request);
        if ($return == ' ' && curl_errno($request) == 0) {
            $response_data['submit_res'] = true;
            $_SESSION['csme_step_num'] = "3";
        } else if (curl_errno($request) == 0) {
            $xml = simplexml_load_string($return);
            if (!empty($xml->error)) {
                $response_data['submit_res'] = false;
                $response_data['Curlgetinfo'] = (string) $xml->error;
            } else {
                $response_data['submit_res'] = false;
                $response_data['Curlgetinfo'] = curl_getinfo($request);
            }
        } else {
            $response_data['submit_res'] = false;
            $response_data['Curlgetinfo'] = curl_getinfo($request);
        }

        // $result = json_decode(json_encode(simplexml_load_string($return)), true);

        curl_close($request);

        //   $response = json_encode( $response_data );
        //   header( "Content-Type: application/json" );
        return $response_data;
    }

    private function createJob() {
        $response_data = Array();
        //////////////////////start
        $username = $_SESSION['username_s'];
        $password = $_SESSION['password_s'];
        $url = get_option('o_platform_url') . "/users/current.xml";
        $process = curl_init($url);
        curl_setopt($process, CURLOPT_HEADER, 1);
        curl_setopt($process, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($process, CURLOPT_VERBOSE, 1);

        $return = curl_exec($process);

        $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
        $header = substr($return, 0, $header_size);
        $body = substr($return, $header_size);

        curl_close($process);

        $xml_user = (array) simplexml_load_string($body);
        $software_id = get_option('o_software');
        if ($xml_user['user-role-name'] != 'admin') {

            $url = get_option('o_platform_url') . '/licenses.xml';
            $process = curl_init($url);
            curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
            curl_setopt($process, CURLOPT_TIMEOUT, 30);
            curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($process, CURLOPT_VERBOSE, 1);
            $return = curl_exec($process);
            $xml = (array) simplexml_load_string($return);
            curl_close($process);
            if (count($xml) > 1) {
                unset($xml['@attributes']);
                if (is_array($xml['license'])) {
                    foreach ($xml['license'] as $license) {
                        $license = (array) $license;
                        if ($license['user-id'] == $xml_user['id'] && $license['software-id'] == $software_id) {
                            $skip_user_software = 1;
                        }
                    }
                } else {
                    $license = (array) $xml['license'];
                    if ($license['user-id'] == $xml_user['id'] && $license['software-id'] == $software_id) {
                        $skip_user_software = 1;
                    }
                }
                if (!isset($skip_user_software)) {
                    $crete_license = $this->create_license($xml_user['id'], $software_id);
                }
            } else {

                $crete_license = $this->create_license($xml_user['id'], $software_id);
            }
            // $array_data = simplexml_load_string($return);
            $url = get_option('o_platform_url') . '/accesses.xml';
            $process = curl_init($url);
            curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
            curl_setopt($process, CURLOPT_TIMEOUT, 30);
            curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($process, CURLOPT_VERBOSE, 1);
            $return = curl_exec($process);
            $xml = (array) simplexml_load_string($return);
            curl_close($process);
            $resource_id = get_option('o_resource');
            $region_id = get_option('o_region');
            if (count($xml) > 1) {
                unset($xml['@attributes']);
                if (is_array($xml['access'])) {
                    foreach ($xml['access'] as $access) {
                        $access = (array) $access;
                        if ($access['user-id'] == $xml_user['id'] && $access['region-id'] == $region_id && $access['resource-account-id'] == $resource_id) {
                            $skip_user_access = 1;
                        }
                    }
                } else {
                    $access = (array) $xml['access'];
                    if ($access['user-id'] == $xml_user['id'] && $access['region-id'] == $region_id && $access['resource-account-id'] == $resource_id) {
                        $skip_user_access = 1;
                    }
                }
                if (!isset($skip_user_access)) {
                    $crete_license = $this->create_accesses($xml_user['id'], $region_id, $resource_id);
                }
            } else {
                $crete_license = $this->create_accesses($xml_user['id'], $region_id, $resource_id);
            }
        }

        ///////////////////////end
        $url = get_option('o_platform_url') . "/jobs.xml";
        $arg_string = $_POST['args'];
        $name = $_SESSION['username_s'];
        list ($u_name, $trash) = explode('@', $name, 2);
        $date = new DateTime();
        $u_name .= $date->getTimestamp();

        $software_id = get_option('o_software');
        $executable_id = get_option('o_executable');
        $resource_id = get_option('o_resource');
        $region_id = get_option('o_region');
        $instance_type_id = get_option('o_instance_type');

        $post_xml = "<job>
                            <name>$u_name</name>
                            <software_id>$software_id</software_id>
                            <executable_id>$executable_id</executable_id>
                            <resource_id>$resource_id</resource_id>
                            <region_id>$region_id</region_id>
                            <instance_type_id>$instance_type_id</instance_type_id>
                            <previous_job_id></previous_job_id>
                            <nodes>1</nodes>
                            <argument_string>$arg_string</argument_string>
                            <stdout_file_name>output</stdout_file_name>
                            <stderr_file_name>error</stderr_file_name>
                            <archive_output>false</archive_output>
                            <start_immediately>true</start_immediately>
                            <permanently_running>false</permanently_running>
                            <no_execution>false</no_execution>
                            <description></description>
                            <requested_instance_id></requested_instance_id>
                            <output-mask></output-mask>
                        </job>";


        $process = curl_init($url);

        curl_setopt($process, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
        curl_setopt($process, CURLOPT_POST, 1);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
        curl_setopt($process, CURLOPT_POSTFIELDS, $post_xml);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, 1);

        $return = curl_exec($process);

        $array_data = json_decode(json_encode(simplexml_load_string($return)), true);
        $_SESSION['job_id_s'] = $array_data['id'];
        $_SESSION['job_name_s'] = $array_data['name'];

        if (curl_errno($process) == 0) {
            $response_data['create_res'] = true;
        } else {
            $response_data['create_res'] = false;
            $response_data['Curlgetinfo'] = curl_getinfo($process);
        }

        curl_close($process);
        return $response_data;
    }

    /*
     * New fuction to create accesses and licenses 16/08/2017
     */

    private function create_accesses($user_id, $region_id, $resource_id) {
        $username = get_option('o_email');
        $password = get_option('o_password');
        $url = get_option('o_platform_url') . "/resources/" . $resource_id . ".xml";
        $process = curl_init($url);

        curl_setopt($process, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

        $return = curl_exec($process);
        curl_close($process);
        $xml = (array) simplexml_load_string($return);

        $post_xml = "<access>
                            <active>true</active>
                            <region-id>$region_id</region-id>
                            <resource-account-id>" . $xml['default-account-id'] . "</resource-account-id>
                            <user-id>$user_id</user-id>
                            <do_not_create_if_exists>true</do_not_create_if_exists>
                        </access>";

        $url = get_option('o_platform_url') . "/accesses.xml";
        $process = curl_init($url);

        curl_setopt($process, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($process, CURLOPT_POST, 1);
        curl_setopt($process, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
        curl_setopt($process, CURLOPT_POSTFIELDS, $post_xml);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, 1);

        $return = curl_exec($process);
        $array_data = json_decode(json_encode(simplexml_load_string($return)), true);
        return 1;
    }

    private function create_license($user_id, $software_id) {
        $username = get_option('o_email');
        $password = get_option('o_password');

        $post_xml = "<license>
                            <active>true</active>
                            <software-id>$software_id</software-id>
                            <user-id>$user_id</user-id>
                        </license>";

        $url = get_option('o_platform_url') . "/licenses.xml";
        $process = curl_init($url);

        curl_setopt($process, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($process, CURLOPT_POST, 1);
        curl_setopt($process, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
        curl_setopt($process, CURLOPT_POSTFIELDS, $post_xml);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, 1);

        $return = curl_exec($process);

        $array_data = json_decode(json_encode(simplexml_load_string($return)), true);
        return 1;
    }

    private function login($username, $password) {
        $url = get_option('o_platform_url') . "/users/current.xml";
        $process = curl_init($url);
        //curl_setopt($process, CURLOPT_HTTPHEADER, array('Content-Type: application/xml', $additionalHeaders));
        curl_setopt($process, CURLOPT_HEADER, 1);
        curl_setopt($process, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($process, CURLOPT_VERBOSE, 1);

        $return = curl_exec($process);

        $header_size = curl_getinfo($process, CURLINFO_HEADER_SIZE);
        $header = substr($return, 0, $header_size);
        $body = substr($return, $header_size);

        curl_close($process);

        $xml = simplexml_load_string($body);

        if (!empty($xml) && !empty($xml->id)) {
            $this->username = $username;
            $this->password = $password;
            $this->userId = $xml->id;

            $_SESSION['username_s'] = $username;
            $_SESSION['password_s'] = $password;
            $_SESSION['jobs_array_s'] = array();
            $_SESSION['uid_s'] = json_decode(json_encode((array) $xml->id), TRUE)[0];

            return true;
        } else {
            return false;
        }
    }

    public function run() {
        $this->loader->run();
    }

    public function get_plugin_name() {
        return $this->plugin_name;
    }

    public function get_loader() {
        return $this->loader;
    }

    public function get_version() {
        return $this->version;
    }

    public function my_session_start() {

        if (!session_id()) {
            session_start();

            //$_SESSION['csme_step_num'] = "1";
            //$_SESSION['csme_sid'] = session_id();
        } else {
            //$_SESSION['csme_step_num'] = "9";
            //$_SESSION['csme_sid'] = session_id();
        }
    }

    function my_session_end() {
        $this->logout_actions();
    }

    public function cancel_job() {
        $job_id = $_SESSION['job_id_s'];
        $url = get_option('o_platform_url') . "/jobs/" . $job_id . "/stop.xml";
        $response_data = Array();
        $body = Array();

        $request = curl_init($url);

        curl_setopt($request, CURLOPT_USERPWD, $_SESSION['username_s'] . ":" . $_SESSION['password_s']);
        curl_setopt($request, CURLOPT_CUSTOMREQUEST, "PUT");
        curl_setopt($request, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($request, CURLOPT_POSTFIELDS, http_build_query($body));

        $return = curl_exec($request);
    }

}
